#include "filed.h"
#include "main.h"
#include "externs.h"
#include "shmem.h"
#include "dbq.h"
#include "db.h"
#include "log.h"
#include "flags.h"
#include "conf.h"
#include <string.h>
#include <stdio.h>
#include <signal.h>

void handle_db(void) {
	/* this is practically its own main() */
	struct db *db;

	shmem_db_ready();
	db_setup_signals();
	
	if(!(sys_flags & FLAG_UP)) sys_flags += FLAG_UP;
	
	while(sys_flags & FLAG_UP) {
		/* to shut this thread down,
		we send it a term anyway, so why not? */
	
		while((db = shmem_db_get_next())) {
			/* returns a pointer to a static struct */
			if(db) dlog(DEBUG_CODE,"db thread received: %d",db->id);
			d_process_db(db);
			/* acknowlage */
		}
		ack_ping();
		usleep(1);
	}
	dlog(DEBUG_THREAD,"database thread closing");
	shmem_shutdown();
	exit(0);
}

void d_process_db(struct db *db) {
	char buf[1024];

	switch(db->filter) {
		case 1:
			dlog(DEBUG_MAIN,"test ordered: %s",db->query);
			sprintf(buf,"OK NOOP %s",db->query);
			db->output = strdup(buf);
			db->flags = SHMEM_CLAIMED + SHMEM_PROCESSED + SHMEM_DONE;
			shmem_db_set(db);
			break;
		default:
			sprintf(buf,"ERR wtf");
			db->output = strdup(buf);
			db->flags = SHMEM_CLAIMED + SHMEM_PROCESSED + SHMEM_ERR + SHMEM_DONE;
			shmem_db_set(db);
			break;
	}
	
	return;
}

void ack_ping(void) {
	struct msg *msg = filed->msg;
	
	if(msg->master_flags & SHMEM_FLAG_DBPING)
		msg->master_flags -= SHMEM_FLAG_DBPING;
}

void db_setup_signals(void) {
	signal(SIGTERM,dbsigterm);
	signal(SIGHUP,dbsighup);
	signal(SIGINT,SIG_IGN);
	signal(SIGCHLD,SIG_IGN);
}

void dbsigterm(int sig) {
	sys_flags -= FLAG_UP;
	dlog(DEBUG_THREAD,"DB thread received SIGTERM");
	return;
}

void dbsighup(int sig) {
	read_cf();
	dlog(DEBUG_THREAD,"DB thread received SIGHUP");
	return;
}

